#ifndef HEAP_DEBUGGER_H
#define HEAP_DEBUGGER_H
#include "mallocator.h"
#include <unordered_map>
#include "CallStack.h"
#include <set>
#include <queue>

class HeapDebugger
{


  struct PageData
  {
    void* pageAddress;
    size_t releaseAllocationId;
    size_t numPages;
  };

  struct AllocationData
  {
    void* addr;
    size_t size;
    CallStack callStack;
    PageData* page;
    bool isArray;
    const char* file;
    unsigned int line;
  };

  std::unordered_map<void*, 
                     AllocationData*, 
                     std::hash<void*>, 
                     std::equal_to<void*>, 
                     Mallocator<std::pair<void*, AllocationData*>>> allocationData;
  std::queue<PageData*, std::deque<PageData*, Mallocator<PageData*>>> decomittedPages;
  
  static HeapDebugger* instance;
  size_t currentAllocationId;

  HeapDebugger* prevHeapDebugger;
  const char* filename;

  AllocationData* PageAlignedAllocate(size_t size, bool isArray, 
                                      const char* file, unsigned int line);
  void DecommitPage(PageData* page);
public:
  HeapDebugger(const char* filename = nullptr);
  ~HeapDebugger();

  void* AllocateNoThrow(size_t size, bool isArray, const char* file, unsigned int line);

  void FreeNoThrow(void* ptr, bool isArray);

  void* Allocate(size_t size, bool isArray, const char* file, unsigned int line);

  void Free(void* ptr, bool isArray);

  void WriteLeaks();

  static HeapDebugger* GetInstance();
  void ReleaseOldPages();
};

void * operator new(size_t size);

void * operator new(size_t size, const std::nothrow_t&) throw ();

void * operator new[](size_t size);

void * operator new[](size_t size, const std::nothrow_t&) throw ();

void * operator new(size_t size, const char* file, unsigned int line);

void * operator new(size_t size, const std::nothrow_t&, const char* file, unsigned int line) throw ();

void * operator new[](size_t size, const char* file, unsigned int line);

void * operator new[](size_t size, const std::nothrow_t&, const char* file, unsigned int line) throw ();

void operator delete(void * ptr) throw ();

void operator delete(void * ptr, const std::nothrow_t&) throw ();


void operator delete[](void * ptr) throw ();

void operator delete[](void * ptr, const std::nothrow_t&) throw ();

#ifndef DONT_DEFINE_NEW
#define new new(__FILE__, __LINE__)
#endif

#endif